package tiw5.cineservice.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

@Entity
@NamedQueries({
		@NamedQuery(name = "findSeanceByDate", query = "SELECT s FROM Seance s WHERE s.date BETWEEN :start AND :end"),
		@NamedQuery(name = "findSeanceByDateAndFilm", query = "SELECT s FROM Seance s JOIN s.film f WHERE (s.date BETWEEN :start AND :end) AND f.id = :filmId") })
@XmlRootElement
public class Seance {

	@Id
	@GeneratedValue
	@XmlAttribute
	private long id;
	@ManyToOne
	private Film film;
	@ManyToOne
	private Salle salle;
	@Temporal(TemporalType.TIMESTAMP)
	@XmlElement
	private Date date;
	@OneToMany(mappedBy = "seance", cascade = CascadeType.REMOVE)
	private Collection<Reservation> reservations = new ArrayList<Reservation>();

	protected Seance() {
	}

	public Seance(Film film, Salle salle, Date date) {
		this.film = film;
		this.salle = salle;
		this.date = date;
	}

	public Collection<Reservation> reserver(int nombre) {
		ArrayList<Reservation> result = new ArrayList<Reservation>(nombre);
		if (salle.getCapacite() >= reservations.size() + nombre) {
			for (int i = 0; i < nombre; ++i) {
				result.add(new Reservation(this));
			}
			reservations.addAll(result);
		}
		return result;
	}

	@XmlTransient
	public Collection<Reservation> getReservations() {
		return reservations;
	}

	public void supprimeReservations() {
		reservations.clear();
	}

	public long getId() {
		return id;
	}

	@XmlElement
	public Salle getSalle() {
		return salle;
	}
	
	public void setSalle(Salle salle) {
		this.salle = salle;
	}
}
