package tiw5.cineservice.service;

import static tiw5.cineservice.service.CineConstants.TARGET_NAMESPACE;

import java.util.Calendar;
import java.util.Collection;

import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.springframework.transaction.annotation.Transactional;

import tiw5.cineservice.model.Reservation;
import tiw5.cineservice.model.Seance;

@WebService(targetNamespace = TARGET_NAMESPACE)
public class Reservations {

	@PersistenceContext
	private EntityManager em;

	public @WebResult(name = "seance", targetNamespace = TARGET_NAMESPACE) Collection<Seance> seances(
			@WebParam(name = "jour", targetNamespace = TARGET_NAMESPACE) Calendar jour) {
		Calendar from = (Calendar) jour.clone();
		from.set(Calendar.HOUR_OF_DAY, 0);
		from.set(Calendar.MINUTE, 0);
		from.set(Calendar.SECOND, 0);
		Calendar to = Calendar.getInstance();
		to.setTimeInMillis(from.getTimeInMillis() + 1000 * 60 * 60 * 24);
		Query q = em.createNamedQuery("findSeanceByDate");
		q.setParameter("start", from.getTime());
		q.setParameter("end", to.getTime());
		@SuppressWarnings("unchecked")
		Collection<Seance> result = q.getResultList();
		return result;
	}

	@Transactional
	public @WebResult(name = "reservations", targetNamespace = TARGET_NAMESPACE) ResultatReservation reserver(
			@WebParam(name = "seanceId", targetNamespace = TARGET_NAMESPACE) long seanceId,
			@WebParam(name = "nombrePlaces", targetNamespace = TARGET_NAMESPACE) int nombre)
			throws PlusDePlaceException {
        Seance s = em.find(Seance.class,seanceId);
        if (s == null) {
            throw new PlusDePlaceException(
                    "La seance cherchée n'a pas été trouvée");
        } else {
            Collection<Reservation> reservations = s.reserver(nombre);
            if (reservations == null || reservations.size() == 0) {
                throw new PlusDePlaceException("Il reste seulement "
                        + (s.getSalle().getCapacite() - s.getReservations()
                        .size()) + " places");
            } else {
                for(Reservation r : reservations) {
                    em.persist(r);
                }
                return new ResultatReservation(reservations);
            }
        }
	}

}
